#!/usr/bin/env python3

# Copyright (c) 2019 Jon Evans <jon@craftyjon.com>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

# This utility compares two netlists generated by Eeschema and reports on
# connectivity differences between them.  This only is looking at connectivity!
# Things like the components section, the visual format of the file, and the
# sorting of entries is not compared.

import argparse
import json
import os
import sexpdata
import sys


def extract_nets(sexpr):
    # Starts with ( export (version N) (...) )
    for idx, key in enumerate(sexpr):
        # print("{}: {}".format(idx, key))
        if isinstance(key, list):
            if isinstance(key[0], sexpdata.Symbol) and key[0].value() == "nets":
                return key[1:]
    return None


def unpack(sexpr):
    ret = {}
    for net in sexpr:
        code = net[1][1]
        name = net[2][1]
        if isinstance(name, sexpdata.Symbol):
            name = name.value()
        name = str(name)

        members = []
        if len(net) < 4:
            continue

        for node in net[3:]:
            ref = node[1][1]
            pin = node[2][1]

            if isinstance(ref, sexpdata.Symbol):
                ref = ref.value()

            if isinstance(pin, sexpdata.Symbol):
                pin = pin.value()

            ref = str(ref)
            pin = str(pin)

            members.append((ref, pin))

        members.sort()
        ret[name] = members

    return ret


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Compare KiCad netlists')
    parser.add_argument('first_netlist')
    parser.add_argument('second_netlist')
    parser.add_argument('--quiet', action='store_true')

    args = parser.parse_args()

    fn_a = args.first_netlist
    fn_b = args.second_netlist

    with open(fn_a, 'r') as f:
        a = sexpdata.load(f)

    with open(fn_b, 'r') as f:
        b = sexpdata.load(f)

    # Extract the nets portion
    a = extract_nets(a)
    b = extract_nets(b)

    if a is None:
        if not args.quiet:
            print("Could not read nets from {}".format(fn_a))
        sys.exit(-1)

    if b is None:
        if not args.quiet:
            print("Could not read nets from {}".format(fn_b))
        sys.exit(-1)

    nets_a = unpack(a)
    nets_b = unpack(b)

    sa = set(nets_a.keys())
    sb = set(nets_b.keys())

    only_a = sa - sb
    only_b = sb - sa
    both = sa & sb

    if len(only_a) == len(only_b) == 0:
        if not args.quiet:
            print("{} and {} are identical".format(fn_a, fn_b))
        sys.exit(0)

    if not args.quiet:
        print("A: {}\nB: {}".format(fn_a, fn_b))

        changed_header = False

        for net_name in sorted(both):
            if nets_a[net_name] != nets_b[net_name]:
                if not changed_header:
                    print("\nChanged nets:\n")
                    changed_header = True

                print("{}: {} => {}".format(net_name, nets_a[net_name],
                                            nets_b[net_name]))

        discards_a = set()
        discards_b = set()

        renamed_header = False

        for net_name in sorted(only_a):
            for candidate in only_b:
                if nets_a[net_name] == nets_b[candidate]:
                    if not renamed_header:
                        print("\nRenamed nets (no connection changes):\n")
                        renamed_header = True

                    print("{} => {}".format(net_name, candidate))
                    discards_a.add(net_name)
                    discards_b.add(candidate)

        only_a.difference_update(discards_a)
        only_b.difference_update(discards_b)

        if len(only_a) > 0:
            print("\nOnly in {}:\n".format(fn_a))
            print('\n'.join(["{}: {}".format(el, nets_a[el])
                            for el in sorted(only_a)]))

        if len(only_b) > 0:
            print("\nOnly in {}:\n".format(fn_b))
            print('\n'.join(["{}: {}".format(el, nets_b[el])
                            for el in sorted(only_b)]))

    sys.exit(1)



import os

i = 0

with open( 'CKJ_lib.bak' ) as f:
    for line in f:
        tok = line.split()
        if len(tok) == 0:
            print( '' )
            continue

        if tok[0] == 'P':
            tok[3:-2:2] = [str( int( t ) * 1.5 ) for t in tok[3:-2:2]]
        elif tok[0] == 'X' and int( tok[3] ) > 0:
            tok[3] = '1450'

        print( ' '.join( tok ) )
        
        


"""Unit-test parts of the CONNECTIVITY_DATA Python API."""

import unittest

import pcbnew


class TestConnectivity(unittest.TestCase):
    """Test that calls on BOARD.Connectivity() are functional."""

    def setUp(self):
        """Setup shared attributes."""
        self.pcb = pcbnew.LoadBoard("data/complex_hierarchy.kicad_pcb")
        self.connectivity = self.pcb.GetConnectivity()
        self.nets = self.pcb.GetNetsByName()

    def test_get_connectivity_returns_connectivity_data_object(self):
        """Verify: GetConnectivity() returns a CONNECTIVITY_DATA object."""
        connectivity = self.pcb.GetConnectivity()
        self.assertEqual(type(connectivity).__name__, "CONNECTIVITY_DATA")

    def test_get_connected_pads_on_track_returns_iterable_of_pads(self):
        """Verify: GetConnectedPads(track) returns an iterable of pads."""
        net = list(self.nets.values())[1]
        tracks = list(self.pcb.TracksInNet(net.GetNetCode()))
        track_with_pad = tracks[1]
        pads = self.connectivity.GetConnectedPads(track_with_pad)
        self.assertGreater(len(pads), 0)
        self.assertTrue(all(pad.GetClass() == "PAD" for pad in pads))

    def test_get_connected_tracks_returns_iterable_of_tracks(self):
        """Verify: GetConnectedTracks(track) returns an iterable of tracks."""
        net = list(self.nets.values())[1]
        net_tracks = self.pcb.TracksInNet(net.GetNetCode())
        net_track = list(net_tracks)[0]
        connected_tracks = self.connectivity.GetConnectedTracks(net_track)
        self.assertGreater(len(connected_tracks), 1)
        self.assertTrue(
            all(track.GetClass() == "PCB_TRACK" for track in connected_tracks)
        )



import code
import unittest
import pcbnew
import pdb

class TestPCBLoad(unittest.TestCase):

    def setUp(self):
        self.pcb = pcbnew.LoadBoard("data/complex_hierarchy.kicad_pcb")

    def test_pcb_load(self):
        self.assertNotEqual(self.pcb,None)

    def test_pcb_track_count(self):
        tracks = list(self.pcb.GetTracks())
        self.assertEqual(len(tracks),361)

    def test_pcb_modules(self):
        modules = list(self.pcb.GetFootprints())
        self.assertEqual(len(modules), 72)

    def test_pcb_module_references(self):
        board_refs = list(module.GetReference() for
                          module in self.pcb.GetFootprints())

        known_refs = [u'P1', u'P3', u'C2', u'C1', u'D1', u'Q3', u'Q5', u'Q7',
                      u'Q6', u'Q1', u'Q2', u'Q4', u'Q8', u'P2', u'U1', u'U4',
                      u'P4', u'P5', u'P6', u'U3', u'R9', u'R15', u'RV1', u'RV2',
                      u'C3', u'C4', u'C5', u'C6', u'C7', u'C8', u'C9', u'D2',
                      u'D3', u'D4', u'D5', u'D6', u'D7', u'R3', u'R4', u'R5',
                      u'R6', u'R7', u'R8', u'R10', u'R11', u'R12', u'R13',
                      u'R14', u'R16', u'R17', u'R18', u'R19', u'R20', u'R21',
                      u'R22', u'MIRE', u'C10', u'C11',
                      u'U2', u'C14', u'C12', u'R23', u'R24', u'D9', u'D8', u'R25',
                      u'R26', u'R27', u'R28']

        for ref in known_refs:
            self.assertTrue(ref in board_refs)

    def test_pcb_netcount(self):
        self.assertEqual(self.pcb.GetNetCount(),51)

    def test_pcb_shapes(self):
        drawings = list(self.pcb.GetDrawings())
        edge_cuts = [d for d in drawings if d.GetLayer() == pcbnew.Edge_Cuts]
        coordinates = [[list(edge.GetStart()), list(edge.GetEnd())] for edge in edge_cuts]
        expected_coordinates = [
            [[88265000, 51816000], [188595000, 51816000]],
            [[88265000, 131826000], [88265000, 51816000]],
            [[188595000, 51816000], [188595000, 131826000]],
            [[188595000, 131826000], [88265000, 131826000]]
        ]
        self.assertEqual(sorted(coordinates), sorted(expected_coordinates))

    def test_pcb_text(self):
        drawings = list(self.pcb.GetDrawings())
        text = [d for d in drawings if d.GetClass() == "PTEXT"]
        self.verify_text(text[0], 173355000, 68453000, pcbnew.F_Cu,
                         u'Actionneur\nPiezo New Amp\nV02')
        self.verify_text(text[1], 176149000, 64643000, pcbnew.B_Cu,
                         u'Actionneur\nPiezo New Amp\nV02')

    def test_text_as_segments(self):
        footprint = self.pcb.FindFootprintByReference("U1")
        reference = footprint.Reference()
        segments = [[p.x, p.y] for p in reference.TransformToSegmentList()]
        expected_segments = [
            [141901333, 69196857], [143340666, 69196857], [143340666, 69196857],
            [143510000, 69142428], [143510000, 69142428], [143594666, 69088000],
            [143594666, 69088000], [143679333, 68979142], [143679333, 68979142],
            [143679333, 68761428], [143679333, 68761428], [143594666, 68652571],
            [143594666, 68652571], [143510000, 68598142], [143510000, 68598142],
            [143340666, 68543714], [143340666, 68543714], [141901333, 68543714],
            [143679333, 67400714], [143679333, 68053857], [143679333, 67727285],
            [141901333, 67727285], [141901333, 67727285], [142155333, 67836142],
            [142155333, 67836142], [142324666, 67945000], [142324666, 67945000],
            [142409333, 68053857]
        ]
        self.assertEqual(segments, expected_segments)

    def verify_text(self, text, x, y, layer, s):
        self.assertEquals(list(text.GetPosition()), [x, y])
        self.assertEquals(text.GetLayer(), layer)
        self.assertEquals(text.GetText(), s)

    #def test_interactive(self):
    #    code.interact(local=locals())

if __name__ == '__main__':
    unittest.main()



import unittest
import pcbnew

class TestTracks(unittest.TestCase):

    def setUp(self):
        self.pcb = pcbnew.LoadBoard("data/tracks_arcs_vias.kicad_pcb")

    def test_tracks(self):
        tracks = [t for t in self.pcb.Tracks() if t.GetClass() == 'PCB_TRACK']
        self.assertEqual(16, len(tracks))
        track = sorted(tracks, key=lambda t: [t.GetStart()[0], t.GetStart()[1]])[0]
        self.assertEqual([27000000, 27585787], [track.GetStart()[0], track.GetStart()[1]])
        self.assertEqual([27000000, 26500000], [track.GetEnd()[0], track.GetEnd()[1]])
        self.assertEqual(250000, track.GetWidth())
        self.assertEqual('McNetty', track.GetNetname())

    def test_arcs(self):
        arcs = [t.Cast() for t in self.pcb.Tracks() if t.GetClass() == 'PCB_ARC']
        self.assertEqual(13, len(arcs))
        arc = sorted(arcs, key=lambda t: [t.GetStart()[0], t.GetStart()[1]])[0]
        self.assertEqual( [29414200, 26500000], [arc.GetCenter()[0], arc.GetCenter()[1]] )
        self.assertEqual([1800, 2250], [round(arc.GetArcAngleStart()), round(arc.GetArcAngleEnd())])
        self.assertEqual( 2414200, round( arc.GetRadius() ) )

    def test_vias(self):
        vias = [t.Cast() for t in self.pcb.Tracks() if t.GetClass() == 'PCB_VIA']
        self.assertEqual(2, len(vias))
        via = sorted(vias, key=lambda t: [t.GetStart()[0], t.GetStart()[1]])[0]
        self.assertEqual([29000000, 41000000], [via.GetStart()[0], via.GetStart()[1]])
        self.assertEqual(400000, via.GetDrillValue())



import unittest
import pcbnew

class TestPads(unittest.TestCase):

    def setUp(self):
        self.pcb = pcbnew.LoadBoard("data/custom_pads.kicad_pcb")

    def test_custom_pads_outline(self):
        custom_pad1 = self.pcb.FindFootprintByReference("SB1").Pads()[0]
        expected_polygons = [[
            [1000000, 0],
            [500000, 750000],
            [-500000, 750000],
            [-500000, -750000],
            [500000, -750000]
        ]]
        # test accessor without layer
        polygon_set = custom_pad1.GetCustomShapeAsPolygon()
        self.assertEqual(expected_polygons, self.parse_polygon_set(polygon_set))
        # test accessor with layer
        polygon_set = custom_pad1.GetCustomShapeAsPolygon(pcbnew.F_Cu)
        self.assertEqual(expected_polygons, self.parse_polygon_set(polygon_set))

    def parse_polygon_set(self, polygon_set):
        result = []
        for polygon_index in range(polygon_set.OutlineCount()):
            outline = polygon_set.Outline(polygon_index)
            parsed_outline = []
            for point_index in range(outline.PointCount()):
                point = outline.CPoint(point_index)
                parsed_outline.append([point.x, point.y])
            result.append(parsed_outline)
        return result



import unittest
import platform
import sys
import argparse

if platform.python_version() < '2.7':
    unittest = __import__('unittest2')
else:
    import unittest

try:
    import xmlrunner
    have_xml = True
except ImportError:
    have_xml = False


if __name__ == '__main__':

    parser = argparse.ArgumentParser(
            description='Test suit for KiCad Python functions')
    parser.add_argument('--xml', action="store", type=str,
                        help='Output XML test results to the given directory')

    args = parser.parse_args()

    if args.xml and not have_xml:
        print("XML test reporting not available")
        print("Install the xmlrunner package.")
        sys.exit(2)

    testsuite = unittest.TestLoader().discover('testcases', pattern="*.py")

    if args.xml:
        # Dump XML results to the right directory
        runner = xmlrunner.XMLTestRunner(output=args.xml)
    else:
        # Use a normal text runner
        runner = unittest.TextTestRunner(verbosity=100)

    results = runner.run(testsuite)

    # Return an error code if any of the testsuite tests fail
    if not results.wasSuccessful():
        sys.exit(1)
